# -*- coding: utf-8 -*-
#  kakasi.py
#
# Copyright 2011-2021 Hiroshi Miura <miurahr@linux.com>
#
import enum
from typing import Dict, List, Tuple

import jaconv

from .kanji import JConv
from .properties import Ch
from .scripts import A2, H2, IConv, K2, Sym2


class PyKakasiException(Exception):
    pass


class UnknownCharacterException(PyKakasiException):
    pass


class _TYPE(enum.Enum):
    KANJI = 1
    KANA = 2
    HIRAGANA = 3
    SYMBOL = 4
    ALPHA = 5


class _ACTION(enum.Enum):
    NOBUFOUT_AND_OUTPUT_CURRENT_AND_NEXT = 1
    BUFOUT_AND_SKIP_CURRENT = 2
    BUFOUT_AND_OUTPUT_CURRENT_AND_NEXT = 3
    PUT_AND_NEXT = 4
    BUFOUT_AND_NEXT = 5
    DO_NOTHING = 6


class Kakasi:
    """Kakasi is a conversion class for Japanese text."""

    def __init__(self):
        self._jconv = JConv()
        self._iconv = IConv()

    @classmethod
    def normalize(cls, text):
        return jaconv.normalize(text)

    def convert(self, text: str) -> List[Dict[str, str]]:
        """Convert input text to dictionary contains KANA, HIRA and romaji results."""

        if len(text) == 0:
            return [
                {
                    "orig": "",
                    "kana": "",
                    "hira": "",
                    "hepburn": "",
                    "passport": "",
                    "kunrei": "",
                }
            ]

        original_text = ""
        kana_text = ""
        _result = []
        i = 0
        prev_type = _TYPE.KANJI
        action_flag: _ACTION

        while i < len(text):
            if text[i] in Ch.endmark:
                prev_type = _TYPE.SYMBOL
                action_flag = _ACTION.BUFOUT_AND_NEXT
            elif text[i] in Ch.long_symbols:
                # Just follow previous character type
                # we don't change prev_type
                action_flag = _ACTION.PUT_AND_NEXT
            elif K2.isRegion(text[i]):
                if prev_type == _TYPE.KANA:
                    action_flag = _ACTION.PUT_AND_NEXT
                else:
                    action_flag = _ACTION.BUFOUT_AND_NEXT
                prev_type = _TYPE.KANA
            elif H2.isRegion(text[i]):
                if prev_type == _TYPE.HIRAGANA:
                    action_flag = _ACTION.PUT_AND_NEXT
                else:
                    action_flag = _ACTION.BUFOUT_AND_NEXT
                prev_type = _TYPE.HIRAGANA
            elif A2.isRegion(text[i]):
                if prev_type == _TYPE.ALPHA:
                    action_flag = _ACTION.PUT_AND_NEXT
                else:
                    action_flag = _ACTION.BUFOUT_AND_NEXT
                prev_type = _TYPE.ALPHA
            elif Sym2.isRegion(text[i]):
                if prev_type != _TYPE.SYMBOL:
                    action_flag = _ACTION.BUFOUT_AND_NEXT
                else:
                    action_flag = _ACTION.NOBUFOUT_AND_OUTPUT_CURRENT_AND_NEXT
                prev_type = _TYPE.SYMBOL
            elif self._jconv.isRegion(text[i]):
                # flush first
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                t, ln = self._jconv.convert(text[i:], original_text)
                prev_type = _TYPE.KANJI
                if ln > 0:
                    original_text = text[i : i + ln]
                    kana_text = t
                    i += ln
                    action_flag = _ACTION.DO_NOTHING
                else:  # unknown kanji
                    original_text = text[i]
                    kana_text = ""
                    i += 1
                    action_flag = _ACTION.BUFOUT_AND_SKIP_CURRENT
            elif 0xF000 <= ord(text[i]) <= 0xFFFD or 0x10000 <= ord(text[i]) <= 0x10FFFD:
                # PUA: ignore and drop
                prev_type = _TYPE.SYMBOL
                # flush first
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                i += 1
                action_flag = _ACTION.DO_NOTHING
            else:
                # flush first
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                _result.append(self._iconv.convert(text[i], ""))
                i += 1
                action_flag = _ACTION.DO_NOTHING

            # Convert to kana and Output based on flag
            if action_flag == _ACTION.BUFOUT_AND_OUTPUT_CURRENT_AND_NEXT:
                original_text += text[i]
                kana_text += text[i]
                _result.append(self._iconv.convert(original_text, kana_text))
                original_text = ""
                kana_text = ""
                i += 1
            elif action_flag == _ACTION.BUFOUT_AND_NEXT:
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                original_text = text[i]
                kana_text = text[i]
                i += 1
            elif action_flag == _ACTION.PUT_AND_NEXT:
                original_text += text[i]
                kana_text += text[i]
                i += 1
            elif  action_flag == _ACTION.NOBUFOUT_AND_OUTPUT_CURRENT_AND_NEXT:
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                _result.append(self._iconv.convert(text[i], text[i]))
                original_text = ""
                kana_text = ""
                i += 1
            elif action_flag == _ACTION.BUFOUT_AND_SKIP_CURRENT:
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                original_text = ""
                kana_text = ""
                i += 1
            elif action_flag == _ACTION.DO_NOTHING:
                pass
            else:
                pass

        # last word
        if len(original_text) > 0:
            _result.append(self._iconv.convert(original_text, kana_text))

        return _result



# -*- coding: utf-8 -*-
#  kakasi.py
#
# Copyright 2011-2021 Hiroshi Miura <miurahr@linux.com>
#

__license__ = "GPLv3"
__copyright__ = """\
PyKakasi::
    Copyright (C) 2010-2021 Hiroshi Miura and contributors

KAKASI Dictionary::
    Copyright (C) 2010-2021 Hiroshi Miura and contributors

    Copyright (C) 1992 1993 1994 Hironobu Takahashi, Masahiko Sato,
    Yukiyoshi Kameyama, Miki Inooka, Akihiko Sasaki, Dai Ando, Junichi Okukawa,
    Katsushi Sato and Nobuhiro Yamagishi

UniDic::
    PyKakasi re-licenses a part of the unidic under the GPL3+.

    Copyright (c) 2011-2021, The UniDic Consortium

    All rights reserved.

    Unidic is released under any of the GPL2, the LGPL2.1,
    or the 3-clause BSD License.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
__docformat__ = "restructuredtext en"

from .kakasi import Kakasi, PyKakasiException, UnknownCharacterException
from .legacy import (
    InvalidFlagValueException,
    InvalidModeValueException,
    UnknownOptionsException,
    UnsupportedRomanRulesException,
    kakasi,
    wakati,
)

__all__ = [
    "Kakasi",
    "kakasi",
    "wakati",
    "PyKakasiException",
    "UnknownCharacterException",
    "UnsupportedRomanRulesException",
    "UnknownOptionsException",
    "InvalidModeValueException",
    "InvalidFlagValueException",
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import platform
import re
import sys
import warnings
from typing import Any, Optional

import pykakasi

from importlib import metadata as importlib_metadata


class Cli:

    romanvals = {"h": "Hepburn", "k": "Kunrei", "p": "Passport"}

    def __init__(self):
        self.kakasi = None
        self.parser = argparse.ArgumentParser(
            prog="kakasi",
            description="kakasi implementation on Python",
            formatter_class=argparse.RawTextHelpFormatter,
            add_help=True,
        )
        self.parser.add_argument("-v", "--version", action="store_true", help="Show version string")
        self.parser.add_argument(
            "-I", type=argparse.FileType("r"), default=sys.stdin, help="Specify input file (default STDIN)"
        )
        self.parser.add_argument(
            "-O", type=argparse.FileType("w"), default=sys.stdout, help="Specify output file (default STDOUT)"
        )
        self.parser.add_argument("-w", "--wakati", action="store_true", help="Wakati gaki mode")
        self.parser.add_argument(
            "-r",
            "--roman",
            action="store",
            choices=["k", "h", "p"],
            help="Set romanize method; k:kunrei, h:hepburn, or p:passport",
        )
        self.parser.add_argument("-s", "--space", action="store_true", help="Add spacing between tokens")
        self.parser.add_argument("-S", "--separator", action="store", help="Set separator character")
        self.parser.add_argument("-f", "--furigana", action="store_true", help="Furigana output")
        self.parser.add_argument("-C", "--capital", action="store_true", help="Capitalize output")
        self.parser.add_argument("-U", "--upper", action="store_true", help="Upper case output")
        self.parser.add_argument("-E", action="store", choices=["a", "N"], help="Full-width roman characters conversion")
        self.parser.add_argument("-J", action="store", choices=["a", "H", "K", "N"], help="Kanji characters conversion")
        self.parser.add_argument("-K", action="store", choices=["a", "H", "N"], help="Katakana conversion")
        self.parser.add_argument("-H", action="store", choices=["a", "K", "N"], help="Hiragana conversion")
        self.parser.add_argument("-a", action="store", choices=["E", "N"], help="Alphabet conversion")

    def run(self, arg: Optional[Any] = None) -> int:
        outfile = None
        infile = None
        mode = {}
        args = self.parser.parse_args(arg)
        if args.version:
            return self.show_version()
        if args.space:
            mode["s"] = True
        if args.separator is not None:
            mode["S"] = args.separator
        if args.roman is not None:
            mode["r"] = args.roman
        if args.furigana:
            mode["f"] = True
        if args.capital:
            mode["C"] = True
        if args.upper:
            mode["U"] = True
        if args.J is not None and "N" != args.J:
            mode["J"] = args.J
        if args.H is not None and "N" != args.H:
            mode["H"] = args.H
        if args.K is not None and "N" != args.K:
            mode["K"] = args.K
        if args.E is not None and "N" != args.E:
            mode["E"] = args.E
        if args.a is not None and "N" != args.a:
            mode["a"] = args.a
        infile = args.I
        outfile = args.O
        if args.wakati:
            self.kakasi = pykakasi.wakati()
            self.do_it(mode, infile, outfile)
        else:
            self.kakasi = pykakasi.kakasi()
            self.do_it(mode, infile, outfile)
        return 0

    def show_version(self):
        dist = importlib_metadata.distribution("pykakasi")
        py_version = platform.python_version()
        py_impl = platform.python_implementation()
        py_build = platform.python_compiler()
        print("pykakasi: version {} on Python {} [{} {}]".format(dist.version, py_version, py_impl, py_build))
        return 0

    def do_it(self, mode, infile, outfile):
        for k, v in mode.items():
            self.kakasi.setMode(k, v)
        converter = self.kakasi.getConverter()
        for line in infile:
            outfile.write(converter.do(line))


def cli_main():
    warnings.simplefilter("ignore", category=DeprecationWarning)
    sys.exit(Cli().run())


if __name__ == "__main__":
    sys.argv[0] = re.sub(r"(-script\.pyw|\.exe)?$", "", sys.argv[0])
    sys.exit(cli_main())
